# iscream::XMLParser

# iscream::XMLParser decodes a string of XML into a
# has in a very similar way to the Java version in
# the i-scream CMS.

package iscream::XMLParser;

use strict;
use XML::Parser;

my @tagstack;
my %xmlhash;

sub parse {
    my($xml) = @_;
    %xmlhash = ();

    my $parser = new XML::Parser;
    
    $parser->setHandlers(Char => \&char_handler,
                         Start => \&start_handler,
                         End => \&end_handler);
    
    eval { $parser->parse($xml) };
    
    return ($@, %xmlhash);
}

sub char_handler {
    my ($p, $value) = @_;
    &add_value($value);
}

sub start_handler {
    shift;
    my($tag) = shift;
    push(@tagstack, $tag);
    my($name) = shift;
    my($val) = shift;
    while(defined $name && defined $val) {
       push(@tagstack, "attributes.$name");
       &add_value($val);
       pop(@tagstack);
       $name = shift;
       $val = shift;
    }
}

sub end_handler {
    pop(@tagstack); # safe to not check, parser will do it
}

sub add_value {
    my($value) = @_;
    my($tag) = "";
    foreach my $item (@tagstack) {
        $tag = $tag . $item . ".";
    }
    $tag =~ s/^(.*)\.$/$1/;
    $xmlhash{$tag} = $value;
}

1;
